/*
 * Created on 26 sept. 2005
 *
 */
package net.sf.emis.ide.wizards;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.emis.ide.IdePlugin;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.internal.ide.IDEWorkbenchPlugin;


public abstract class AbstractSetPrefsPage extends WizardPage {
    
    protected HashMap varMap = new HashMap();
    
    public AbstractSetPrefsPage(String id, String title, String description) {
        super(id);
        setTitle(title);
        setDescription(description);
     }

    protected Text createTextField(Composite parent, final String varName, String msg) {
        
        Label label = new Label(parent, SWT.NONE);
        label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END)); 
        label.setText(msg);
        Text text = new Text(parent, SWT.SINGLE);
        text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));   
        if (varMap.get(varName) != null) {
            text.setText(varMap.get(varName).toString());
        } else {
            text.setText("");
            varMap.put(varName, "");
        }
        
        text.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                String text = ((Text)e.getSource()).getText();
                varMap.put(varName, text);
                setPageComplete(validatePage());
            }
        });
        return text;
        
    }
    
    protected Text createChooseDirField(Composite parent, String varName, String label) {

        final Text text = createTextField(parent, varName, label);
        Button button = new Button(parent, SWT.PUSH);
        button.setText("Browse");
        button.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                DirectoryDialog dialog = new DirectoryDialog(text.getShell());
                dialog.setMessage("Selection du rpertoire");
                String selectedDirectory = dialog.open();
                if (selectedDirectory != null) {
                    text.setText(selectedDirectory);
                }
            }
        });
        return text;        
        
    }   
    
    protected Text createChooseFileField(Composite parent, String varName, String label) {

        final Text text = createTextField(parent, varName, label);
        Button button = new Button(parent, SWT.PUSH);
        button.setText("Browse");
        button.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                FileDialog dialog = new FileDialog(text.getShell());
                //dialog.setMessage("Selection du Fichier");
                String selectedFile = dialog.open();
                if (selectedFile != null) {
                    text.setText(selectedFile);
                }
            }
        });
        return text;        
        
    }      
    
    protected Group createGroup(Composite parent, String label, int numColumns) {
        Group group = new Group(parent, SWT.NONE);
        GridLayout groupLayout = new GridLayout();
        groupLayout.numColumns = numColumns;
        group.setLayout(groupLayout);        
        group.setText(label);  
        group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        return group;
    }
    
    public abstract void createFileSet(final IFolder folder);
    
    protected ISchedulingRule createRule(IResource resource) {
        IResource parent = resource.getParent();
        while (parent != null) {
            if (parent.exists())
                return resource.getWorkspace().getRuleFactory().createRule(resource);
            resource = parent;
            parent = parent.getParent();
        }
        return resource.getWorkspace().getRoot();
    }    
    
    protected InputStream getResourceStream(String filePath) {
        
        try {
            URL url = Platform.resolve(IdePlugin.getDefault().getBundle().getEntry(filePath));
            FileInputStream fis = new FileInputStream(url.getPath());
            return fis;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }
    
    /**
     * Creates a file resource given the file handle and contents.
     *
     * @param fileHandle the file handle to create a file resource with
     * @param contents the initial contents of the new file resource, or
     *   <code>null</code> if none (equivalent to an empty stream)
     * @param monitor the progress monitor to show visual progress with
     * @exception CoreException if the operation fails
     * @exception OperationCanceledException if the operation is canceled
     */
    protected void createFile(IFile fileHandle, InputStream contents,
            IProgressMonitor monitor) throws CoreException {
        if (contents == null)
            contents = new ByteArrayInputStream(new byte[0]);

        try {
            // Create a new file resource in the workspace

                IPath path = fileHandle.getFullPath();
                IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
                int numSegments= path.segmentCount();
                if (numSegments > 2 && !root.getFolder(path.removeLastSegments(1)).exists()) {
                    // If the direct parent of the path doesn't exist, try to create the
                    // necessary directories.
                    for (int i= numSegments - 2; i > 0; i--) {
                        IFolder folder = root.getFolder(path.removeLastSegments(i));
                        if (!folder.exists()) {
                            folder.create(false, true, monitor);
                        }
                    }
                }
                fileHandle.create(contents, false, monitor);

        } catch (CoreException e) {
            // If the file already existed locally, just refresh to get contents
            if (e.getStatus().getCode() == IResourceStatus.PATH_OCCUPIED)
                fileHandle.refreshLocal(IResource.DEPTH_ZERO, null);
            else
                throw e;
        }

        if (monitor.isCanceled())
            throw new OperationCanceledException();
    }

    
    protected void replaceContent(IFile ifile) {
        
        try {
            File iFile = new File(ifile.getLocation().toString());
            FileReader fileReader = new FileReader(iFile);
            BufferedReader reader = new BufferedReader(fileReader);
            File oFile = new File(iFile.getParent() + File.separator + iFile.getName().substring(1));
            oFile.createNewFile();
            FileWriter fileWriter = new FileWriter(oFile);
            BufferedWriter writer = new BufferedWriter(fileWriter);

            String line = reader.readLine();
            while (line != null) {
                line = parseLine(line);
                writer.write(parseLine(line) + "\n");
                line = reader.readLine();
            }
            writer.flush();
            writer.close();
            fileWriter.close();
            reader.close();
            fileReader.close();
            iFile.delete();
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        
    }
    
    protected String parseLine(String line) {
        
        // dtecter les variables 
        Pattern p = Pattern.compile(".*\\$\\{(.*)\\}.*", Pattern.DOTALL);
        Matcher m = p.matcher(line);
        m.useAnchoringBounds(false);
        while (m.lookingAt()) {
            String varName = m.group(1);
            String varValue = (varMap.get(varName) == null) ? "\\${"+varName+"}":varMap.get(varName).toString();
            int offset = m.start(1);
            line = line.replaceAll("\\$\\{" + varName + "\\}", varValue);
            m = p.matcher(line);
            if (offset < line.length())
                m = m.region(0, offset);
            m.useAnchoringBounds(false);
        }       
        return line;        
        
    }
    

    
    /**
     * Creates a file resource handle for the file with the given workspace path.
     * This method does not create the file resource; this is the responsibility
     * of <code>createFile</code>.
     *
     * @param filePath the path of the file resource to create a handle for
     * @return the new file resource handle
     * @see #createFile
     */
    protected IFile createFileHandle(IPath filePath) {
        return IDEWorkbenchPlugin.getPluginWorkspace().getRoot().getFile(filePath);
    }

    protected boolean validatePage() {
        boolean valid = true;

        if (varMap.containsValue("")) {
            setMessage("Veuillez renseigner tous les champs !");
            return false;
        }

        if (valid) {
            setMessage(null);
            setErrorMessage(null);
        }
        return valid;
    }    
    
}
