/*
 * Created on 26 sept. 2005
 *
 */
package net.sf.emis.ide.wizards;

import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaModelStatus;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.util.CoreUtility;
import org.eclipse.jdt.internal.ui.wizards.NewWizardMessages;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.BuildPathsBlock;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.ContainerGenerator;
import org.eclipse.ui.internal.ide.IDEWorkbenchMessages;
import org.eclipse.ui.internal.ide.IDEWorkbenchPlugin;


public class NewModuleSetPrefsPage extends AbstractSetPrefsPage {

    public static String ID = "NewModuleSetPrefsPage";
    
    //private HashMap varMap = new HashMap();
    
    public NewModuleSetPrefsPage(String pageName) {
        super(ID, pageName, "Les paramtre FTP permettent le dploiement des ressources.");
     }

    public void createControl(Composite parent) {

        initializeDialogUnits(parent);
        // top level group
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setFont(parent.getFont());
        composite.setLayout(new GridLayout());
        composite.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_FILL
                | GridData.HORIZONTAL_ALIGN_FILL));
        
        Group group = createGroup(composite, "Set", 2);
        createTextField(group, "group.jarName", "Jar Name : ");
        createTextField(group, "group.displayName", "Display Name : ");
        createTextField(group, "group.description", "Description : ");
        group = createGroup(composite, "Paramtres FTP", 2);
        createTextField(group, "ftp.host", "Ftp Host : ");
        createTextField(group, "ftp.port", "Ftp Port : ");
        createTextField(group, "ftp.dir", "Ftp Dir : ");
        createTextField(group, "ftp.user", "Ftp User : ");
        createTextField(group, "ftp.pwd", "Ftp Password : ");
        createTextField(group, "ftp.passive", "Ftp Mode Passive (yes|no) : ");
        group = createGroup(composite, "Classpath", 3);
        createChooseDirField(group, "jboss.libs.client.path", "Bibliothques client JBoss : ");
        createChooseDirField(group, "xdoclet.libs.path", "Bibliothques XDoclet : ");
        createChooseFileField(group, "emis.lib.commons.jar", "Jar eMis commons : ");
        createChooseDirField(group, "emis.lib.xtra.path", "eMis extras : ");
        
        setControl(composite);
        setPageComplete(false);
        
    }
    

    
    public void createFileSet(final IFolder folder) {
        
        WorkspaceModifyOperation op = new WorkspaceModifyOperation(createRule(folder)) {
            public void execute(IProgressMonitor monitor) throws CoreException {
                try {
                    monitor.beginTask(IDEWorkbenchMessages.WizardNewFolderCreationPage_progress, 2000);
                    ContainerGenerator generator = new ContainerGenerator(folder.getFullPath());
                    generator.generateContainer(new SubProgressMonitor(monitor, 1000));
                    
                    InputStream is = getResourceStream("templates/module-set-build.xml");
                    IFile buildHandle = createFileHandle(folder.getFullPath().append("build.xml"));
                    createFile(buildHandle, is, new SubProgressMonitor(monitor, 400));

                    is = getResourceStream("templates/emis-module-set.properties");
                    IFile propertiesHandle = createFileHandle(folder.getFullPath().append(".emis-module-set.properties"));
                    createFile(propertiesHandle, is, new SubProgressMonitor(monitor, 300));
                    replaceContent(propertiesHandle);
                    
                    is = getResourceStream("templates/home-local.xdt");
                    IFile aliasHandle = createFileHandle(folder.getFullPath().append("home-local.xdt"));
                    createFile(aliasHandle, is, new SubProgressMonitor(monitor, 300));
                    
                    //createSourceFolder(folder, monitor);
                    
                    folder.refreshLocal(IResource.DEPTH_ONE, null);
                    
                } catch (Throwable th) {
                    th.printStackTrace();
                } finally {
                    monitor.done();
                }
            }
        };
        
        try {
            getContainer().run(true, true, op);
            createSourceFolder(folder);
            
        } catch (InterruptedException e) {
            e.printStackTrace();
            return;
        } catch (InvocationTargetException e) {
            e.printStackTrace();
            if (e.getTargetException() instanceof CoreException) {
                ErrorDialog
                        .openError(
                                getContainer().getShell(), // Was Utilities.getFocusShell()
                                IDEWorkbenchMessages.WizardNewFolderCreationPage_errorTitle,
                                null, // no special message
                                ((CoreException) e.getTargetException())
                                        .getStatus());
            } else {
                // CoreExceptions are handled above, but unexpected runtime exceptions and errors may still occur.

                IDEWorkbenchPlugin.log(getClass(),
                        "createNewFolder()", e.getTargetException()); //$NON-NLS-1$
                MessageDialog
                        .openError(
                                getContainer().getShell(),
                                IDEWorkbenchMessages.WizardNewFolderCreationPage_internalErrorTitle, NLS.bind(IDEWorkbenchMessages.WizardNewFolder_internalError, e.getTargetException().getMessage()));
            }
            return; // ie.- one of the steps resulted in a core exception
        } catch (OperationCanceledException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
    }
    
    private void createSourceFolder(IFolder folder) throws OperationCanceledException, CoreException {
        
        System.out.println("1");

        boolean fIsProjectAsSourceFolder= false;
        IPath path= folder.getFullPath().append("src");
        IJavaProject fCurrJProject = null;
        IClasspathEntry[] fEntries = null;
        IPath fOutputLocation = null;
        IProject project= folder.getProject();
        
        try {
            if (project.hasNature(JavaCore.NATURE_ID)) {
                fCurrJProject= JavaCore.create(project);
                fEntries= fCurrJProject.getRawClasspath();
                fOutputLocation= fCurrJProject.getOutputLocation();
            } else return;
        } catch (JavaModelException e) {
            // TODO Auto-generated catch block
            System.out.println("a");
            e.printStackTrace();
        } catch (CoreException e) {
            System.out.println("b");
            // TODO Auto-generated catch block
            e.printStackTrace();
        }  
        
        System.out.println("2");
        
        
        IPath projPath= fCurrJProject.getProject().getFullPath();
        
        ArrayList newEntries= new ArrayList(fEntries.length + 1);
        int projectEntryIndex= -1;
        
        for (int i= 0; i < fEntries.length; i++) {
            IClasspathEntry curr= fEntries[i];
            if (curr.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
                if (path.equals(curr.getPath())) {
                    System.out.println("c");
                    //fRootStatus.setError(NewWizardMessages.NewSourceFolderWizardPage_error_AlreadyExisting); 
                    return;
                }
                if (projPath.equals(curr.getPath())) {
                    projectEntryIndex= i;
                }   
            }
            newEntries.add(curr);
        }        
        
        System.out.println("3");

        IClasspathEntry newEntry= JavaCore.newSourceEntry(path);
        
        Set modified= new HashSet();                
        
        if (projectEntryIndex != -1) {
            fIsProjectAsSourceFolder= true;
            newEntries.set(projectEntryIndex, newEntry);
        } else {
            newEntries.add(JavaCore.newSourceEntry(path));
        }

        IClasspathEntry[] fNewEntries= (IClasspathEntry[]) newEntries.toArray(new IClasspathEntry[newEntries.size()]);
        IPath fNewOutputLocation= fOutputLocation;

        System.out.println("4");

        IJavaModelStatus status= JavaConventions.validateClasspath(fCurrJProject, fNewEntries, fNewOutputLocation);
        if (!status.isOK()) {
            if (fOutputLocation.equals(projPath)) {
                fNewOutputLocation= projPath.append("bin");
                IStatus status2= JavaConventions.validateClasspath(fCurrJProject, fNewEntries, fNewOutputLocation);
                if (status2.isOK()) {
                    if (fIsProjectAsSourceFolder) {
                        //fRootStatus.setInfo(Messages.format(NewWizardMessages.NewSourceFolderWizardPage_warning_ReplaceSFandOL, fNewOutputLocation.makeRelative().toString())); 
                    } else {
                        //fRootStatus.setInfo(Messages.format(NewWizardMessages.NewSourceFolderWizardPage_warning_ReplaceOL, fNewOutputLocation.makeRelative().toString())); 
                    }
                    System.out.println("d");
                    return;
                }
            }
            System.out.println("e");
            //fRootStatus.setError(status.getMessage());
            return;
        } else if (fIsProjectAsSourceFolder) {
            System.out.println("f");
            //fRootStatus.setInfo(NewWizardMessages.NewSourceFolderWizardPage_warning_ReplaceSF); 
            return;
        } 
        
        
        //IPath projPath= fCurrJProject.getProject().getFullPath();
        if (fOutputLocation.equals(projPath) && !fNewOutputLocation.equals(projPath)) {
            if (BuildPathsBlock.hasClassfiles(fCurrJProject.getProject())) {
                if (BuildPathsBlock.getRemoveOldBinariesQuery(getShell()).doQuery(projPath)) {
                    BuildPathsBlock.removeOldClassfiles(fCurrJProject.getProject());
                }
            }
        }   
        
        IProgressMonitor monitor= new NullProgressMonitor();
        monitor.beginTask(NewWizardMessages.NewSourceFolderWizardPage_operation, 3); 
        try {
            IFolder srcFolder= fCurrJProject.getProject().getFolder(folder.getProjectRelativePath().append("src"));
            System.out.println(srcFolder.getLocation().toString());
            if (!srcFolder.exists()) {
                System.out.println("createFolder");
                CoreUtility.createFolder(srcFolder, true, true, new SubProgressMonitor(monitor, 1));           
            } 
            fCurrJProject.setRawClasspath(fNewEntries, fNewOutputLocation, monitor);
        } finally {
            monitor.done();
        }
        //fCreatedRoot= fCurrJProject.getPackageFragmentRoot(folder);        
        
    }
 

}
