/*
 * Created on 26 sept. 2005
 *
 */
package net.sf.emis.ide.wizards;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.ContainerGenerator;
import org.eclipse.ui.internal.ide.IDEWorkbenchMessages;
import org.eclipse.ui.internal.ide.IDEWorkbenchPlugin;
import org.eclipse.ui.internal.ide.IIDEHelpContextIds;
import org.eclipse.ui.internal.ide.misc.ResourceAndContainerGroup;

public class NewSetMainPage extends WizardPage implements Listener {
    
    public static String ID = "NewSetMainPage";
    
    private static final int SIZING_CONTAINER_GROUP_HEIGHT = 250;
    
    private IStructuredSelection currentSelection;
    private IFolder newFolder;
    
    private ResourceAndContainerGroup resourceGroup;
    private Button advancedButton;
    

    public NewSetMainPage(String pageName, String description, IStructuredSelection selection) {
        super(ID);
        setTitle(pageName);
        setDescription(description);
        this.currentSelection = selection;
    }

    public void createControl(Composite parent) {
        try{
        initializeDialogUnits(parent);
        // top level group
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setFont(parent.getFont());
        composite.setLayout(new GridLayout());
        composite.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_FILL
                | GridData.HORIZONTAL_ALIGN_FILL));

        PlatformUI.getWorkbench().getHelpSystem()
                .setHelp(composite, IIDEHelpContextIds.NEW_FOLDER_WIZARD_PAGE);

        resourceGroup = new ResourceAndContainerGroup(
                composite,
                this,
                IDEWorkbenchMessages.WizardNewFolderMainPage_folderName, IDEWorkbenchMessages.WizardNewFolderMainPage_folderLabel, false, SIZING_CONTAINER_GROUP_HEIGHT);
        resourceGroup.setAllowExistingResources(false);

        initializePage();
        validatePage();
        // Show description on opening
        setErrorMessage(null);
        setMessage(null);
        setControl(composite);
        } catch (Throwable th) {
            th.printStackTrace();
        }
    }
    
    /**
     * Creates a folder resource given the folder handle.
     *
     * @param folderHandle the folder handle to create a folder resource for
     * @param monitor the progress monitor to show visual progress with
     * @exception CoreException if the operation fails
     * @exception OperationCanceledException if the operation is canceled
     */
    protected void createFolder(IFolder folderHandle, IProgressMonitor monitor)
            throws CoreException {
        try {
            // Create the folder resource in the workspace
            // Update: Recursive to create any folders which do not exist already
            if (!folderHandle.exists()) {

  
                    IPath path = folderHandle.getFullPath();
                    IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
                    int numSegments= path.segmentCount();
                    if (numSegments > 2 && !root.getFolder(path.removeLastSegments(1)).exists()) {
                        // If the direct parent of the path doesn't exist, try to create the
                        // necessary directories.
                        for (int i= numSegments - 2; i > 0; i--) {
                            IFolder folder = root.getFolder(path.removeLastSegments(i));
                            if (!folder.exists()) {
                                folder.create(false, true, monitor);
                            }
                        }
                    }
                    folderHandle.create(false, true, monitor);

            }
        } catch (CoreException e) {
            // If the folder already existed locally, just refresh to get contents
            if (e.getStatus().getCode() == IResourceStatus.PATH_OCCUPIED)
                folderHandle.refreshLocal(IResource.DEPTH_INFINITE,
                        new SubProgressMonitor(monitor, 500));
            else
                throw e;
        }

        if (monitor.isCanceled())
            throw new OperationCanceledException();
    }
    

    /**
     * Creates a folder resource handle for the folder with the given workspace path.
     * This method does not create the folder resource; this is the responsibility
     * of <code>createFolder</code>.
     *
     * @param folderPath the path of the folder resource to create a handle for
     * @return the new folder resource handle
     * @see #createFolder
     */
    protected IFolder createFolderHandle(IPath folderPath) {
        return IDEWorkbenchPlugin.getPluginWorkspace().getRoot().getFolder(
                folderPath);
    }

    /**
     * Creates a new folder resource in the selected container and with the selected
     * name. Creates any missing resource containers along the path; does nothing if
     * the container resources already exist.
     * <p>
     * In normal usage, this method is invoked after the user has pressed Finish on
     * the wizard; the enablement of the Finish button implies that all controls on
     * this page currently contain valid values.
     * </p>
     * <p>
     * Note that this page caches the new folder once it has been successfully
     * created; subsequent invocations of this method will answer the same
     * folder resource without attempting to create it again.
     * </p>
     * <p>
     * This method should be called within a workspace modify operation since
     * it creates resources.
     * </p>
     *
     * @return the created folder resource, or <code>null</code> if the folder
     *    was not created
     */
    public IFolder createNewFolder() {
        if (newFolder != null)
            return newFolder;

        // create the new folder and cache it if successful
        final IPath containerPath = resourceGroup.getContainerFullPath();
        //System.out.println("resourceGroup.getResource()=" + resourceGroup.getResource());
        IPath newFolderPath = containerPath.append(resourceGroup.getResource());
        final IFolder newFolderHandle = createFolderHandle(newFolderPath);

        WorkspaceModifyOperation op = new WorkspaceModifyOperation(createRule(newFolderHandle)) {
            public void execute(IProgressMonitor monitor) throws CoreException {
                try {
                    monitor.beginTask(IDEWorkbenchMessages.WizardNewFolderCreationPage_progress, 2000);
                    ContainerGenerator generator = new ContainerGenerator(containerPath);
                    generator.generateContainer(new SubProgressMonitor(monitor, 1000));
                    createFolder(newFolderHandle, new SubProgressMonitor(monitor, 1000));
                    
                    /*InputStream is = getResourceStream("templates/alias-set-build.xml");
                    IFile buildHandle = createFileHandle(newFolderHandle.getFullPath().append("build.xml"));
                    createFile(buildHandle, is, new SubProgressMonitor(monitor, 250));
                    
                    IFile propertiesHandle = createFileHandle(newFolderHandle.getFullPath().append("emis-alias-set.properties"));
                    createFile(propertiesHandle, null, new SubProgressMonitor(monitor, 250));

                    is = getResourceStream("templates/emis-alias-set.xml");
                    IFile aliasHandle = createFileHandle(newFolderHandle.getFullPath().append("emis-alias-set.xml"));
                    createFile(aliasHandle, is, new SubProgressMonitor(monitor, 250));*/

                } catch (Throwable th) {
                    th.printStackTrace();
                } finally {
                    monitor.done();
                }
            }
        };

        try {
            getContainer().run(true, true, op);
        } catch (InterruptedException e) {
            return null;
        } catch (InvocationTargetException e) {
            if (e.getTargetException() instanceof CoreException) {
                ErrorDialog
                        .openError(
                                getContainer().getShell(), // Was Utilities.getFocusShell()
                                IDEWorkbenchMessages.WizardNewFolderCreationPage_errorTitle,
                                null, // no special message
                                ((CoreException) e.getTargetException())
                                        .getStatus());
            } else {
                // CoreExceptions are handled above, but unexpected runtime exceptions and errors may still occur.

                IDEWorkbenchPlugin.log(getClass(),
                        "createNewFolder()", e.getTargetException()); //$NON-NLS-1$
                MessageDialog
                        .openError(
                                getContainer().getShell(),
                                IDEWorkbenchMessages.WizardNewFolderCreationPage_internalErrorTitle, NLS.bind(IDEWorkbenchMessages.WizardNewFolder_internalError, e.getTargetException().getMessage()));
            }
            return null; // ie.- one of the steps resulted in a core exception
        }

        newFolder = newFolderHandle;

        return newFolder;
    }
    

    /*public String getFolderName() {
        return resourceGroup.getResource();
    }*/
    
    /**
     * Returns the scheduling rule to use when creating the resource at
     * the given container path. The rule should be the creation rule for
     * the top-most non-existing parent.
     * @param resource The resource being created
     * @return The scheduling rule for creating the given resource
     * @since 3.1
     */
    private ISchedulingRule createRule(IResource resource) {
        IResource parent = resource.getParent();
        while (parent != null) {
            if (parent.exists())
                return resource.getWorkspace().getRuleFactory().createRule(resource);
            resource = parent;
            parent = parent.getParent();
        }
        return resource.getWorkspace().getRoot();
    }
    
    public void handleEvent(Event ev) {
        setPageComplete(validatePage());
    }

    protected void initializePage() {
        Iterator it = currentSelection.iterator();
        if (it.hasNext()) {
            Object next = it.next();
            IResource selectedResource = null;
            if (next instanceof IResource) {
                selectedResource = (IResource) next;
            } else if (next instanceof IAdaptable) {
                selectedResource = (IResource) ((IAdaptable) next)
                        .getAdapter(IResource.class);
            }
            if (selectedResource != null) {
                if (selectedResource.getType() == IResource.FILE)
                    selectedResource = selectedResource.getParent();
                if (selectedResource.isAccessible())
                    resourceGroup.setContainerFullPath(selectedResource
                            .getFullPath());
            }
        }

        setPageComplete(false);
    }    

    /**
     * Returns whether this page's controls currently all contain valid 
     * values.
     *
     * @return <code>true</code> if all controls are valid, and
     *   <code>false</code> if at least one is invalid
     */
    protected boolean validatePage() {
        boolean valid = true;

        if (!resourceGroup.areAllValuesValid()) {
            // if blank name then fail silently
            if (resourceGroup.getProblemType() == ResourceAndContainerGroup.PROBLEM_RESOURCE_EMPTY
                    || resourceGroup.getProblemType() == ResourceAndContainerGroup.PROBLEM_CONTAINER_EMPTY) {
                setMessage(resourceGroup.getProblemMessage());
                setErrorMessage(null);
            } else {
                setErrorMessage(resourceGroup.getProblemMessage());
            }
            valid = false;
        }

        /*IStatus linkedResourceStatus = null;
        if (valid) {
            linkedResourceStatus = validateLinkedResource();
            if (linkedResourceStatus.getSeverity() == IStatus.ERROR)
                valid = false;
        }*/
        // validateLinkedResource sets messages itself
        if (valid) {
            setMessage(null);
            setErrorMessage(null);
        }
        return valid;
    }
    
    public void setVisible(boolean visible) {
        super.setVisible(visible);
        if (visible)
            resourceGroup.setFocus();
    }    
    
}
